#include "ndestyleanimations.h"

#include <QAbstractItemView>
#include <QComboBox>
#include <QCheckBox>
#include <QDial>
#include <QGroupBox>
#include <QHeaderView>
#include <QLineEdit>
#include <QProgressBar>
#include <QRadioButton>
#include <QScrollBar>
#include <QSpinBox>
#include <QTextEdit>
#include <QToolBox>
#include <QToolButton>
#include <QPushButton>
#include <QTreeView>

namespace NdeStyle
{

Animations::Animations( QObject* parent ):
    QObject( parent )
{
    registerEngine( _widgetStateEngine = new WidgetStateEngine( this ) );
    registerEngine( _widgetStateWidthEngine = new WidgetStateWidthEngine( this ) );
}
Animations::~Animations(){

}
WidgetStateEngine& Animations::widgetStateEngine( void ) const{
    return *_widgetStateEngine;
}

WidgetStateWidthEngine& Animations::widgetStateWidthEngine( void ) const{
    return *_widgetStateWidthEngine;
}


void Animations::registerWidget( QWidget* widget ) const
{

    if( !widget ) return;
    //注册button动画
//    if( qobject_cast<QAbstractButton *>(widget) ) {
//        _widgetStateEngine->registerWidget( widget);
//        _widgetStateWidthEngine->registerWidget( widget);
//    }
    if( qobject_cast<QPushButton *>(widget) ) {
        _widgetStateEngine->registerWidget( widget);
        _widgetStateWidthEngine->registerWidget( widget);
    }
    if( qobject_cast<QRadioButton *>(widget) ) {
        _widgetStateEngine->registerWidget( widget);
        _widgetStateWidthEngine->registerWidget( widget);
    }
    //注册tabbar动画
    if( qobject_cast<QTabBar *>(widget) ) {
        _widgetStateEngine->registerWidget( widget);
        //_widgetStateWidthEngine->registerWidget( widget);
    }
    if( qobject_cast<QCheckBox *>(widget)) {
        _widgetStateEngine->registerWidget( widget);
        _widgetStateWidthEngine->registerWidget( widget);
    }
    if( qobject_cast<QSlider *>(widget) ) {
        _widgetStateEngine->registerWidget( widget);
    }
    if( qobject_cast<QLineEdit *>(widget) ) {
        _widgetStateWidthEngine->registerWidget( widget);
    }
    if( qobject_cast<QProgressBar *>(widget) ) {
        _widgetStateWidthEngine->registerWidget( widget);
    }
    return;
}

void Animations::unregisterWidget( QWidget* widget ) const
{
    if( !widget ) return;
    foreach( const BaseEngine::Pointer& engine, _engines )
    { if( engine && engine.data()->unregisterWidget( widget ) ) break; }
}

void Animations::unregisterEngine( QObject* object )
{
    int index( _engines.indexOf( qobject_cast<BaseEngine*>(object) ) );
    if( index >= 0 ) _engines.removeAt( index );
}

void Animations::registerEngine( BaseEngine* engine )
{
    _engines.append( engine );
    connect( engine, SIGNAL(destroyed(QObject*)), this, SLOT(unregisterEngine(QObject*)) );
}

}
