#include "ndestylewidgetstatedata.h"
#include "ndestyle.h"
#include <QEvent>
#include <QMouseEvent>
#include <QDebug>
#include <mutex>
#include <QTabBar>

namespace NdeStyle
{

WidgetStateData::WidgetStateData(QObject* parent, QWidget* target):
    GenericData(parent, target){
    target->installEventFilter(this);
    usecenter = false;
    useradius = false;
    bHovering = false;
    bPressed = false;
    bTabHovering = false;
    bTabPressed = false;
}
WidgetStateData::~WidgetStateData(){

}
bool WidgetStateData::eventFilter(QObject *object, QEvent *event){
    if(target() != object)
        return GenericData::eventFilter( object, event );

    if (object->inherits("QSlider")){
        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            bPressed = true;
            startPress(static_cast<QMouseEvent *>(event), target());
            break;
        case QEvent::MouseButtonRelease:
            bHovering = false;
            startHover(static_cast<QMouseEvent *>(event), target());
            break;
        case QEvent::HoverEnter:
            startHover(static_cast<QMouseEvent *>(event), target());
            break;
        case QEvent::HoverLeave:
            if (bPressed){
                startHover(static_cast<QMouseEvent *>(event), target());
            }
            else{
                bHovering = false;
                endHover(static_cast<QMouseEvent *>(event), target());
            }
            break;
        case QEvent::FocusOut:
            bPressed = false;
            bHovering = false;
            endHover(static_cast<QMouseEvent *>(event), target());
            break;

        default: break;
        }
        return GenericData::eventFilter( object, event );
    }else if (object->inherits("QTabBar")){
        switch( event->type() )
        {
        case QEvent::MouseButtonRelease:
            bTabPressed = false;
            startTabRipple(static_cast<QMouseEvent *>(event), target());
            break;
        case QEvent::HoverLeave:
            if (!bTabHovering && bTabPressed)
                startTabHoverRipple(static_cast<QMouseEvent *>(event), target());
            break;
        }
        return GenericData::eventFilter( object, event );
    }else if (object->inherits("QPushButton")|| object->inherits("QCheckBox")|| object->inherits("QRadioButton")){

        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            startRipple(static_cast<QMouseEvent *>(event), target());
            break;

        default: break;
        }
        return GenericData::eventFilter( object, event );
    }

}

void WidgetStateData::startTabRipple(QMouseEvent *event, QWidget * target){
    removeAllRipple();
    bTabHovering = false;
    bTabPressed = true;

    MaterialRipple *ripple;
    ripple= new MaterialRipple(rippledata.center, target);

    ripple->setRadiusStartValue(0);
    ripple->setRadiusEndValue(static_cast<QTabBar *>(target)->tabRect(static_cast<QTabBar *>(target)->currentIndex()).width()/2);

    ripple->setOpacityStartValue(0.35);
    ripple->setOpacityEndValue(0);

    ripple->radiusAnimation()->setDuration(600);
    ripple->opacityAnimation()->setDuration(1300);

    addRipple(ripple);

    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
}


void WidgetStateData::startTabHoverRipple(QMouseEvent *event, QWidget * target){
    bTabHovering = true;
    MaterialRipple *ripple;
    ripple= new MaterialRipple(rippledata.center, target);

    ripple->setRadiusStartValue(rippledata.radius + 8);
    ripple->setRadiusEndValue(rippledata.radius - 6);

    ripple->setOpacityStartValue(-1);
    ripple->setOpacityEndValue(-1);

    ripple->radiusAnimation()->setDuration(600);
    ripple->opacityAnimation()->setDuration(1300);

    addRipple(ripple);
    cnt = 1;
    connect(ripple, SIGNAL(finished()), this, SLOT(tabHoverShow()));

}

void WidgetStateData::tabHoverShow(){

    MaterialRipple *ripple = qobject_cast<MaterialRipple *>(sender());

    //first in bigger value
    ripple->setRadiusStartValue(rippledata.radius - (6*(cnt % 2)) );

    cnt++;

    ripple->setRadiusEndValue(rippledata.radius - (6*(cnt % 2)));

    ripple->start();

}

void WidgetStateData::startRipple(QMouseEvent *event, QWidget *target){
    QPoint pos;
    qreal radiusEndValue;
    if(NdeStyle::Config::rippleStyle == CenteredRipple)
        pos = target->rect().center();
    else
        pos = event->pos();
    if(NdeStyle::Config::useFixedRippleRadius)
        radiusEndValue = NdeStyle::Config::useFixedRippleRadius;
    else
        radiusEndValue = static_cast<qreal>(target->width());
    MaterialRipple *ripple;

    if (usecenter) {
        ripple= new MaterialRipple(rippledata.center, target);
    } else {
        ripple= new MaterialRipple(pos, target);
    }
    if (useradius){
        ripple->setRadiusEndValue(rippledata.radius*3/4);
    } else {
        ripple->setRadiusEndValue(radiusEndValue);
    }
    ripple->setOpacityStartValue(0.35);
    ripple->radiusAnimation()->setDuration(600);
    ripple->opacityAnimation()->setDuration(1300);

    addRipple(ripple);
}

void WidgetStateData::startPress(QMouseEvent *event, QWidget * target){
    removeAllRipple();
    QPoint pos = event->pos();
    radiusMaxValue = static_cast<qreal>(target->width()) >= static_cast<qreal>(target->height()) ? static_cast<qreal>(target->height()) : static_cast<qreal>(target->width()) ;


    MaterialRipple *ripple = new MaterialRipple(pos, target);

    ripple->setRadiusStartValue(radiusMaxValue / 4);
    ripple->setRadiusEndValue(radiusMaxValue* 5 / 12 - 3);
    ripple->radiusAnimation()->setDuration(100);

    ripple->setOpacityStartValue(1);
    ripple->setOpacityEndValue(1);
    ripple->opacityAnimation()->setDuration(1300);

    addRipple(ripple);
    //connect(ripple, SIGNAL(finished()), this, SLOT(hoverShow()));
}

void WidgetStateData::startHover(QMouseEvent *event, QWidget * target){
    if (bHovering)
        return ;

    removeAllRipple();
    QPoint pos = event->pos();

    radiusMaxValue = static_cast<qreal>(target->width()) >= static_cast<qreal>(target->height()) ? static_cast<qreal>(target->height()) : static_cast<qreal>(target->width()) ;


    MaterialRipple *ripple = new MaterialRipple(pos, target);

    ripple->setRadiusStartValue(radiusMaxValue*5/12);
    ripple->setRadiusEndValue(radiusMaxValue*5/12+2);

    ripple->setOpacityStartValue(0.2);
    ripple->setOpacityEndValue(0.2);

    ripple->radiusAnimation()->setDuration(300);
    ripple->opacityAnimation()->setDuration(1300);

    bHovering = true;
    addRipple(ripple);
    cnt = 1;
    connect(ripple, SIGNAL(finished()), this, SLOT(hoverShow()));
}

void WidgetStateData::endHover(QMouseEvent *event, QWidget * target){
    removeAllRipple();
}

void WidgetStateData::hoverShow(){
    MaterialRipple *ripple = qobject_cast<MaterialRipple *>(sender());

    //first in bigger value
    ripple->setRadiusStartValue(radiusMaxValue*5/12 + (2*(cnt % 2)) );

    cnt++;

    ripple->setRadiusEndValue(radiusMaxValue*5/12 + (2*(cnt % 2)));

    ripple->start();

}

WidgetStateWidthData::WidgetStateWidthData(QObject* parent, QWidget* target):
    GenericWidthData(parent, target){
    target->installEventFilter(this);
    bBusy = false;
    tabWidth = 0;
    bMoving = false;
}
WidgetStateWidthData::~WidgetStateWidthData(){

}
bool WidgetStateWidthData::eventFilter(QObject *object, QEvent *event){
    if(target() != object)
        return GenericWidthData::eventFilter( object, event );
    if (object->inherits("QProgressBar")){
        switch( event->type() )
        {
        case QEvent::Paint:
            if (!bBusy)
                startProgressBusyRipple(static_cast<QMouseEvent *>(event), target());
            break;
        default: break;
        }
    }else if (object->inherits("QCheckBox") || object->inherits("QTreeView") ){
        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            startCheckBoxMarkRipple(static_cast<QMouseEvent *>(event), target());
            break;
        default: break;
        }
    }else if (object->inherits("QRadioButton")){
        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            startRadioButtonRipple(static_cast<QMouseEvent *>(event), target());
            break;
        default: break;
        }
    }/*else if (object->inherits("QTabBar")){
        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            //if (!bMoving)
                startTabLineRipple(static_cast<QMouseEvent *>(event), target());
            break;
        default: break;
        }
    }*/else if (object->inherits("QLineEdit")){
        switch( event->type() )
        {
        case QEvent::MouseButtonPress:
            startRipple(static_cast<QMouseEvent *>(event), target());
            break;
        case QEvent::FocusOut:
            endRipple(static_cast<QMouseEvent *>(event), target());
            break;
        default: break;
        }
    }

    return GenericWidthData::eventFilter( object, event );
}
void WidgetStateWidthData::startTabLineRipple(QMouseEvent *event, QWidget * target){
    //bMoving = true;
    MaterialWidthRipple *ripple = new MaterialWidthRipple(static_cast<qreal>(target->width()), target);
    ripple->widthAnimation()->setDuration(600);
    ripple->setWidthStartValue(0);
    ripple->setWidthEndValue(197);
    addWidthRipple(ripple);
    connect(ripple, SIGNAL(finished()), this, SLOT(test()));
    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
}

void WidgetStateWidthData::test(){
    bMoving = false;
}

void WidgetStateWidthData::setRippleDataTabWidth(qreal width){
    //qCritical()<<"-----------"<<width;
    tabWidth = width;
}

void WidgetStateWidthData::startRadioButtonRipple(QMouseEvent *event, QWidget * target){
    MaterialWidthRipple *ripple = new MaterialWidthRipple(1, target);

    ripple->widthAnimation()->setDuration(600);
    ripple->setWidthStartValue(0);
    ripple->setWidthEndValue(3);
    addWidthRipple(ripple);
    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
}

void WidgetStateWidthData::startCheckBoxMarkRipple(QMouseEvent *event, QWidget *target){
    MaterialWidthRipple *ripple = new MaterialWidthRipple(1, target);

    ripple->widthAnimation()->setDuration(800);
    ripple->setWidthEndValue(1);
    ripple->setWidthStartValue(0);

    addWidthRipple(ripple);
    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
}

void WidgetStateWidthData::startProgressBusyRipple(QMouseEvent *event, QWidget *target){
    bBusy = true;

    qreal lineWidth = static_cast<qreal>(target->width());
    MaterialWidthRipple *ripple = new MaterialWidthRipple(lineWidth, target);

    ripple->setWidthStartValue(0);
    ripple->widthAnimation()->setDuration(600);
    ripple->setWidthEndValue(2);

    addWidthRipple(ripple);
    connect(ripple, SIGNAL(finished()), this, SLOT(endProgressBusyRipple()));
    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
}

void WidgetStateWidthData::endProgressBusyRipple(){
    //MaterialWidthRipple *ripple = qobject_cast<MaterialWidthRipple *>(sender());
    //ripple->destroyRipple();
    bBusy = false;
}

void WidgetStateWidthData::startRipple(QMouseEvent *event, QWidget *target){
    qreal lineWidth = static_cast<qreal>(target->width()) / 2 ;
    MaterialWidthRipple *ripple = new MaterialWidthRipple(lineWidth, target);

    ripple->widthAnimation()->setDuration(300);
    ripple->setWidthEndValue(lineWidth);
    ripple->setWidthStartValue(0);

    addWidthRipple(ripple);

}

void WidgetStateWidthData::endRipple(QMouseEvent *event, QWidget *target){

    removeAllRipple();

//    qreal lineWidth = static_cast<qreal>(target->width()) / 2 ;
//    MaterialWidthRipple *ripple = new MaterialWidthRipple(0, target);

//    ripple->setWidthEndValue(0);
//    ripple->setWidthStartValue(0);
//    ripple->widthAnimation()->setDuration(200);

//    connect(ripple, SIGNAL(finished()), ripple, SIGNAL(destroyRipple()));
//    addWidthRipple(ripple);

}

void WidgetStateData::setRippleDataPoint(QPoint center)
{
    usecenter = true;
    rippledata.center = center;
}

void WidgetStateData::setRippleDataRadios(int radius)
{
    useradius = true;
    rippledata.radius = radius;
}
}
