#include "ndestylewidgetstateengine.h"

#include <QDebug>
namespace NdeStyle
{

WidgetStateEngine::WidgetStateEngine( QObject* parent ): BaseEngine(parent){

}
WidgetStateEngine::~WidgetStateEngine(){

}
bool WidgetStateEngine::registerWidget(QWidget* widget)
{
    if( !widget ) return false;
    if(!m_data.contains( widget ) ) { m_data.insert( widget, new WidgetStateData( this, widget)); }
    connect( widget, SIGNAL(destroyed(QObject*)), this, SLOT(unregisterWidget(QObject*)), Qt::UniqueConnection );
    return true;
}
bool WidgetStateEngine::unregisterWidget(QObject *object){
    if( !object ) return false;
    bool found = false;
    if( m_data.unregisterWidget( object ) ) found = true;
    return found;
}
QList<RippleData> WidgetStateEngine::ripples( const QObject* object ){
    if( DataMap<WidgetStateData>::Value data = this->data( object) ){
        return static_cast< WidgetStateData * >( data.data() )->ripples();
    }else return QList<RippleData>();
}


DataMap<WidgetStateData>::Value WidgetStateEngine::data( const QObject* object)
{
    return m_data.find( object ).data();
}
DataMap<WidgetStateData>& WidgetStateEngine::dataMap()
{
    return m_data;
}

void WidgetStateEngine::setRippleDataPoint(const QObject *widget, QPoint center)
{
    if (m_data.find(widget).data()){
        m_data.find(widget).data()->setRippleDataPoint(center);
    }
}

void WidgetStateEngine::setRippleDataRadius(const QObject *widget, int radius)
{
    if (m_data.find(widget).data()){
        m_data.find(widget).data()->setRippleDataRadios(radius);
    }
}



WidgetStateWidthEngine::WidgetStateWidthEngine( QObject* parent ): BaseEngine(parent){

}
WidgetStateWidthEngine::~WidgetStateWidthEngine(){

}
bool WidgetStateWidthEngine::registerWidget(QWidget* widget)
{
    if( !widget ) return false;
    if(!m_WidthData.contains( widget ) ) { m_WidthData.insert( widget, new WidgetStateWidthData( this, widget)); }
    connect( widget, SIGNAL(destroyed(QObject*)), this, SLOT(unregisterWidget(QObject*)), Qt::UniqueConnection );
    return true;
}
bool WidgetStateWidthEngine::unregisterWidget(QObject *object){
    if( !object ) return false;
    bool found = false;
    if( m_WidthData.unregisterWidget( object ) ) found = true;
    return found;
}

QList<RippleWidthData> WidgetStateWidthEngine::widthRipples( const QObject* object){
    if( DataMap<WidgetStateWidthData>::Value data = this->data( object) ){
        return static_cast< WidgetStateWidthData * >( data.data() )->widthRipples();
    }else return QList<RippleWidthData>();
}

DataMap<WidgetStateWidthData>::Value WidgetStateWidthEngine::data( const QObject* object)
{
    return m_WidthData.find( object ).data();
}
DataMap<WidgetStateWidthData>& WidgetStateWidthEngine::dataMap()
{
    return m_WidthData;
}

void WidgetStateWidthEngine::setRippleDataWidth(const QObject *widget, qreal width)
{
    if (m_WidthData.find(widget).data()){
        m_WidthData.find(widget).data()->setRippleDataTabWidth(width);

    }
}


}
